/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_FLOW_INFO_COMP_QUERY_H_
#define DOCA_FLOW_INFO_COMP_QUERY_H_

#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

#define DOCA_FLOW_INFO_COMP_NAME_STRING_MAX_LEN (256)
#define DOCA_FLOW_INFO_COMP_VALUE_STRING_MAX_LEN (64)
#define DOCA_FLOW_INFO_COMP_FIELDS_MAX (64)

enum doca_flow_info_comp_primitive_type {
	DOCA_FLOW_INFO_COMP_PRIM_TYPE_NONE,
	/**< value type none - value is empty */
	DOCA_FLOW_INFO_COMP_PRIM_TYPE_STR,
	/**< value type string */
	DOCA_FLOW_INFO_COMP_PRIM_TYPE_UINT,
	/**< value type uint */
	DOCA_FLOW_INFO_COMP_PRIM_TYPE_BOOL,
	/**< value type bool */
};

/**
 * @brief one field data
 */
struct doca_flow_info_comp_query_field_data {
	char name[DOCA_FLOW_INFO_COMP_NAME_STRING_MAX_LEN];
	/**< name of exported data */
	enum doca_flow_info_comp_primitive_type type;
	/**< primitive type of exported data */
	union {
		uint64_t value_uint;
		bool value_bool;
		char value_str[DOCA_FLOW_INFO_COMP_VALUE_STRING_MAX_LEN];
	};
	/**< value of exported data */
};

/**
 * @brief data of array fields
 */
struct doca_flow_info_comp_query_data {
	struct doca_flow_info_comp_query_field_data field_data[DOCA_FLOW_INFO_COMP_FIELDS_MAX];
	/**< array of fields */
	uint16_t nr_fields;
	/**< number of exported fields */
};

/**
 * @brief pipe info data
 */
struct doca_flow_comp_info_comp_query_pipe_info {
	struct doca_flow_info_comp_query_data match;
	/**< matches for the pipeline */
	struct doca_flow_info_comp_query_data match_mask;
	/**< match masks for the pipeline */
};

/**
 * @brief data of pipe
 */
struct doca_flow_info_comp_query_pipe_ctx {
	struct doca_flow_info_comp_query_field_data pipe_id;
	/**< uniq pipe id */
	struct doca_flow_info_comp_query_field_data name;
	/**< name of the pipeline */
	struct doca_flow_info_comp_query_field_data type;
	/**< type of pipe */
	struct doca_flow_info_comp_query_field_data domain;
	/**< pipe steering domain */
	struct doca_flow_info_comp_query_field_data is_root;
	/**<  pipeline is root or not */
	struct doca_flow_comp_info_comp_query_pipe_info pipe_info;
	/**< configuration of the pipe */
};

/**
 * @brief mode type information
 */
struct doca_flow_info_comp_mode_type {
	struct doca_flow_info_comp_query_field_data mode_type;
	/**< mode type */
};

/**
 * @brief Get mode type
 *
 * @param mode_type
 * mode type
 * Return 0 on success, non zero on failure.
 */
DOCA_EXPERIMENTAL
int doca_flow_info_comp_get_mode_type(struct doca_flow_info_comp_mode_type *mode_type);

/**
 * @brief query port ids
 *
 * the function fills the array_port_id with actual port_id
 * and sets number of ports to nr_ports, where nr_ports <= size
 *
 * @param array_port_id
 * Pointer to allocated array for collecting ports' id
 * @param size
 * size of allocated array for collecting ports' id
 * @param nr_ports
 * Pointer to nr_ports.
 * Return 0 on success, non zero on failure.
 */
DOCA_EXPERIMENTAL
int doca_flow_info_comp_query_port_ids(uint16_t *array_port_id, uint16_t size, uint16_t *nr_ports);

/**
 * @brief query pipes' id of port
 *
 * @param port_id
 * port_id
 * @param start_from
 * start index
 * @param size
 * size of allocated array for collecting pipes' id
 * @param array_pipe_id
 * Pointer to allocated array for collecting pipes' id
 * @param nr_pipes
 * Pointer to nr_pipes.
 * Return 0 on success, non zero on failure.
 */
DOCA_EXPERIMENTAL
int doca_flow_info_comp_query_pipes_per_port(uint16_t port_id,
					     int start_from,
					     uint32_t size,
					     uint32_t *array_pipe_id,
					     uint32_t *nr_pipes);

/**
 * @brief query pipe info
 *
 * @param port_id
 * port_id
 * @param pipe_id
 * pipe_id
 * @param pipe_ctx
 * Pointer to query pipe's info
 * Return 0 on success, non zero on failure.
 */
DOCA_EXPERIMENTAL
int doca_flow_info_comp_query_pipe_info(uint16_t port_id,
					uint32_t pipe_id,
					struct doca_flow_info_comp_query_pipe_ctx *pipe_ctx);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DOCA_FLOW_INFO_COMP_QUERY_H_ */
